package game;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

/*
 * user interface for the game
 * user inputs tower values, number of players, number of soldiers,
 * number of iterations for the algorithm and mutation probability
 */
@SuppressWarnings("serial")
public class Board extends JFrame {

	private static JButton bPlay;
	private JTextField[] towerValues = new JTextField[10];
	private static JToggleButton userPlayingToggle = new JToggleButton();
	private static JLabel iterationNumLabel;

	
	public Board() {
		this.setResizable(false);
		this.setLocation(200, 125);
		for(int i = 0; i < 10; i++) towerValues[i] = new JTextField(2);

		JLabel title = new JLabel("TOWER DEFENSE", SwingConstants.CENTER);
		title.setFont(new Font("Courier New", Font.ITALIC, 40));
		add(title, BorderLayout.NORTH);
		
		//---------------------------------------------------------
		// main panel in the center
		JPanel gamePanel = new JPanel();
		gamePanel.setLayout(new BorderLayout());
		add(gamePanel, BorderLayout.CENTER);
		
		//----------------------------------------------------------
		// tower pictures and text to enter values on the upper side
		JPanel towerPanel = new JPanel();
		towerPanel.setLayout(new BorderLayout());
		gamePanel.add(towerPanel, BorderLayout.NORTH);
		ImageIcon towerPicture = new ImageIcon(getClass().getResource("tower.jpg"));
		ImageIcon imageIcon = new ImageIcon(towerPicture.getImage().getScaledInstance(60, 60, Image.SCALE_DEFAULT));
		JLabel enterTowerValuesText = new JLabel();  
		JPanel towerPicturesPanel = new JPanel();
		towerPicturesPanel.setLayout(new GridLayout(0,10));
		towerPanel.add(enterTowerValuesText, BorderLayout.NORTH);    
		towerPanel.add(towerPicturesPanel, BorderLayout.CENTER);   

		enterTowerValuesText.setText("  Enter tower values below.");
		for(int i = 0; i < 10; i++) towerPicturesPanel.add(new JLabel(imageIcon));
		//--------------------------------------------------------------
		
		// field to enter tower values in the middle
		JPanel towerValuesContainer = new JPanel();
		gamePanel.add(towerValuesContainer, BorderLayout.CENTER);
		towerValuesContainer.setLayout(new GridLayout(0,1));
		
		// tower values text boxes
		JPanel towerValuesPanel = new JPanel();
		towerValuesPanel.setLayout(new GridLayout(0,10));
		towerValuesContainer.add(towerValuesPanel);
		for(int i = 0; i < 10; i++) {
			JPanel panel = new JPanel();
			towerValuesPanel.add(panel);
			panel.add(towerValues[i]);
		}
		
		//---------------------------------------------------------
		// fields to enter 4 parameters on the lower side
		JPanel inputPanel = new JPanel();
		inputPanel.setLayout(new GridLayout(0,6));
		gamePanel.add(inputPanel, BorderLayout.SOUTH);

		inputPanel.add(new JLabel());inputPanel.add(new JLabel());
		inputPanel.add(new JLabel("Number of players:")); JPanel nopPanel = new JPanel();
		JTextField nopText = new JTextField(3); nopPanel.add(nopText); inputPanel.add(nopPanel);
		inputPanel.add(new JLabel());inputPanel.add(new JLabel());
		inputPanel.add(new JLabel());inputPanel.add(new JLabel());
		inputPanel.add(new JLabel("Number of soldiers:"));JPanel nosPanel = new JPanel();
		JTextField nosText = new JTextField(3); nosPanel.add(nosText); inputPanel.add(nosPanel);
		inputPanel.add(new JLabel());inputPanel.add(new JLabel());
		inputPanel.add(new JLabel());inputPanel.add(new JLabel());
		inputPanel.add(new JLabel("Number of iterations:"));JPanel noiPanel = new JPanel();
		JTextField noiText = new JTextField(3); noiPanel.add(noiText); inputPanel.add(noiPanel);
		inputPanel.add(new JLabel());inputPanel.add(new JLabel());
		inputPanel.add(new JLabel());inputPanel.add(new JLabel());
		inputPanel.add(new JLabel("Mutation probability:"));JPanel mutPanel = new JPanel();
		JTextField mutText = new JTextField(3); mutPanel.add(mutText); inputPanel.add(mutPanel); mutText.setText("0.15");
		inputPanel.add(new JLabel());inputPanel.add(new JLabel());

		//---------------------------------------------------------
		// optional user input and play button on the bottom
		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BorderLayout());
		add(bottomPanel, BorderLayout.SOUTH);
		
		JPanel userPlayingPanel = new JPanel();
		userPlayingPanel.setLayout(new BorderLayout());
		bottomPanel.add(userPlayingPanel, BorderLayout.NORTH);
		userPlayingToggle = new JToggleButton();
		userPlayingToggle.setPreferredSize(new Dimension(20, 20));
		
		userPlayingPanel.add(new JLabel("  Do you want to enter your strategy?  "), BorderLayout.WEST);
		JPanel togglePanel = new JPanel();
		togglePanel.setLayout(new BorderLayout());
		userPlayingPanel.add(togglePanel, BorderLayout.CENTER);
		togglePanel.add(userPlayingToggle, BorderLayout.WEST);
		
		JPanel userInputPanel = new JPanel();
		userInputPanel.setLayout(new BorderLayout());
		userPlayingPanel.add(userInputPanel, BorderLayout.SOUTH);
		
		//*********************************************************
		// user strategy and remaining soldiers
		JTextField userSoldiers[] = new JTextField[10];
		for(int i = 0; i < 10; i++) userSoldiers[i] = new JTextField(2);
		
		JPanel userStrategyPanel = new JPanel();
		userStrategyPanel.setVisible(false);
		userStrategyPanel.setLayout(new BorderLayout());
		userInputPanel.add(userStrategyPanel, BorderLayout.SOUTH);
		
		JPanel enterPanel = new JPanel();
		userStrategyPanel.add(enterPanel, BorderLayout.CENTER);
		enterPanel.setLayout(new GridLayout(0,1));
		//row 0
		JPanel armyPanel = new JPanel();
		armyPanel.setLayout(new GridLayout(0,10));
		enterPanel.add(armyPanel);
		for(int i = 0; i < 10; i++) {
			JPanel panel = new JPanel();
			armyPanel.add(panel);
			panel.add(userSoldiers[i]);
		}
		
		//row 1
		JPanel remainingSoldiersNamePanel = new JPanel();
		userStrategyPanel.add(remainingSoldiersNamePanel, BorderLayout.SOUTH);
		remainingSoldiersNamePanel.setLayout(new BorderLayout());
		
		JPanel remainingSoldiersPanel = new JPanel();
		remainingSoldiersPanel.add(new JLabel("Remaining soldiers:"));
		JLabel remainingSoldiers = new JLabel();
		remainingSoldiers.setBorder(BorderFactory.createLineBorder(new Color(0,250,200), 15));
		remainingSoldiersPanel.add(remainingSoldiers);
		remainingSoldiers.setFont(new Font("Times new Roman", Font.ITALIC, 24));
		
		ImageIcon slikaVojnika = new ImageIcon(getClass().getResource("soldiers.gif"));
		ImageIcon slikaVojnikaScaled = new ImageIcon(slikaVojnika.getImage().getScaledInstance(100,100, Image.SCALE_DEFAULT));
		remainingSoldiersPanel.add(new JLabel(slikaVojnikaScaled));
		
		remainingSoldiersNamePanel.add(remainingSoldiersPanel, BorderLayout.CENTER);
				
		//row 2
		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel("Player name:"));
		JTextField name = new JTextField(30);
		namePanel.add(name);
		
		remainingSoldiersNamePanel.add(namePanel, BorderLayout.SOUTH);
		//***************************************************************
		// Listen for changes in the text to update number of remaining soldiers
				for(int i = 0; i < 10; i++) {
					userSoldiers[i].addFocusListener(new FocusListener() {
						
						@Override
						public void focusLost(FocusEvent e) {
							Integer r = 0;
							String s;
								for(int j = 0; j < 10; j++) {
									String broj = userSoldiers[j].getText();
									try {
										if(!(userSoldiers[j].getText().equals("")))
											Integer.parseInt(broj);
									
										if(!(userSoldiers[j].getText().equals("")) && Integer.parseInt(broj) < 0) {
											userSoldiers[j].setText("0");
											JOptionPane.showMessageDialog(remainingSoldiersNamePanel, "You can't put negative number of soldiers!", "Wrong input", JOptionPane.INFORMATION_MESSAGE);
										}
										if(!(s = userSoldiers[j].getText()).equals(""))
											r += Integer.parseInt(s);
									}catch(Exception ex) {
										userSoldiers[j].setText("0");
										JOptionPane.showMessageDialog(remainingSoldiersNamePanel, "Put a positive integer!", "Wrong input", JOptionPane.INFORMATION_MESSAGE);
									}
								}
							Integer ostalo = Integer.parseInt(nosText.getText()) - r;
							remainingSoldiers.setText(ostalo.toString());
						}

						@Override
						public void focusGained(FocusEvent e) {
						}
					});
				}
		//----------------------------------------------------------------------------
		// enable or disable user strategy input
		userPlayingToggle.addActionListener((e) -> {
			if(nosText.getText().isEmpty()) {
				userPlayingToggle.setSelected(false);
				JOptionPane.showMessageDialog(remainingSoldiersNamePanel, "Enter number of soldiers first!", "No input", JOptionPane.INFORMATION_MESSAGE);
			}
			else {
			if(userPlayingToggle.isSelected()) {
				userStrategyPanel.setVisible(true);
				remainingSoldiers.setText(nosText.getText());
				this.setSize(this.getWidth(), this.getHeight() + 170);
			}
			else {
				userStrategyPanel.setVisible(false);
				this.setSize(this.getWidth(), this.getHeight() - 170);
			} }
			});
		
		//---------------------------------------------------------
		// play button on the bottom
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(0,3));
		bottomPanel.add(buttonPanel, BorderLayout.SOUTH);
		
		buttonPanel.add(new JLabel());

		bPlay = new JButton("Play");
		JPanel playButtonPanel = new JPanel();
		playButtonPanel.add(bPlay);
		
		buttonPanel.add(playButtonPanel);
		bPlay.addActionListener((e) -> {
			String s = "";
			for(int i = 0; i < 10; i++) {
				String value = towerValues[i].getText();
				if(value.equals("")) value = "0";
				s = s + value + " ";
			}
			MainThread.setTowerValues(s);
			
			Integer numOfPlayers = Integer.parseInt(nopText.getText());
			MainThread.setNumOfPlayers(numOfPlayers);
			
			Integer numOfSoldiers = Integer.parseInt(nosText.getText());
			MainThread.setNumOfSoldiers(numOfSoldiers);
			
			Integer numOfIterations = Integer.parseInt(noiText.getText());
			MainThread.setNumOfIterations(numOfIterations);
			
			Float mutationProbability = Float.parseFloat(mutText.getText());
			MainThread.setMutationProbability(mutationProbability);
			
			boolean ready = true;
			
			if(userPlayingToggle.isSelected()) {
				String playerName = name.getText();
				String playerStrategy="";
				for(int i = 0; i < 10; i++) {
					String broj = userSoldiers[i].getText();
					if(broj.equals("")) broj = "0";
					playerStrategy = playerStrategy + broj + " ";
				}
	
				if(remainingSoldiers.getText().equals("0")) {
					Player player = new Player(playerName, playerStrategy);
					MainThread.setUser(player);
					this.revalidate();
				}
				else {
					ready = false;
					Integer ostatak = Integer.parseInt(remainingSoldiers.getText());
					System.out.println(ostatak);
					if(ostatak > 0)
						JOptionPane.showMessageDialog(buttonPanel, "You didn't put all your soldiers in the battle!", "Wrong input", JOptionPane.INFORMATION_MESSAGE);
					else
						JOptionPane.showMessageDialog(buttonPanel, "You put too many soldiers in the battle!", "Wrong input", JOptionPane.INFORMATION_MESSAGE);
	
				}
			}
			
			if(ready) {
				Thread mainThread = new MainThread();
				mainThread.start();
			}
			this.revalidate();
		});
		//---------------------------------------------------------
		iterationNumLabel = new JLabel("Processing iteration number ");
		buttonPanel.add(iterationNumLabel);
		iterationNumLabel.setVisible(false);
		//---------------------------------------------------------
		
		this.revalidate();
	}

	public static void setIterationLabel(int n) {
		if (n == 1) {
			iterationNumLabel.setVisible(true);
			bPlay.setEnabled(false);
		}
		iterationNumLabel.setText("Processing iteration number " + n);
	}
	
	public static void finish() {
		bPlay.setEnabled(true);
		iterationNumLabel.setVisible(false);
	}

	public static boolean isUserPlaying() {
		return userPlayingToggle.isSelected();
	}
	
	public static void main(String[] args) {
		
		SwingUtilities.invokeLater(() -> {
			Board board = new Board();
			board.setTitle("Genetic algorithm in tower defense game");
			board.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			board.pack();
			board.setVisible(true);
			});	
	}
}
