package game;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class GAProcesses {

	/*
	 * tournament selection
	 * 3 players are picked randomly and returned in a set
	 */
	public static Set<Player> selection(Map<Player, Integer> playersResults) {
		
		int numOfPlayers = playersResults.size();
		List<Integer> selectedIndices = new LinkedList<>();
		Set<Player> selectedPlayers = new LinkedHashSet<>();
		
		ArrayList<Integer> list = new ArrayList<Integer>();
		// selection without newly added 10% players
        for(int i = 0; i < numOfPlayers*0.9; i++) {
            list.add(i);
        }
        Collections.shuffle(list);
        for(int i = 0; i < 3; i++) {
        	selectedIndices.add(list.get(i));
        }
        Collections.sort(selectedIndices);
        int br = 0;
        for(Map.Entry<Player, Integer> player : playersResults.entrySet()) {
        	if(selectedIndices.contains(br)) {
        		selectedPlayers.add(player.getKey());
        	}
        	br++;
        }
 
        return selectedPlayers;
	}

	/*
	 * the 2 best strategies from 3 selected become parents
	 * the third strategy gets reassigned by one of three crossover functions
	 */
	public static void crossover(List<Player> players, Set<Player> selectedPlayers) {
		
		Player parent1 = (Player) selectedPlayers.toArray()[0];
		Player parent2 = (Player) selectedPlayers.toArray()[1];
		Player weakestPlayer = (Player) selectedPlayers.toArray()[2];
		
		String[] parent1Strategy = parent1.getStrategy().split(" ");
		String[] parent2Strategy = parent2.getStrategy().split(" ");
		Integer[] newStrategy = new Integer[10];
		
		int crossoverType = (int)(Math.random()*3);
		switch(crossoverType) {
			case(0): newStrategy = crossover1(parent1Strategy, parent2Strategy, newStrategy);
					 break;
			case(1): newStrategy = crossover2(parent1Strategy, parent2Strategy, newStrategy);
			 		 break;
			case(2): newStrategy = crossover3(parent1Strategy, parent2Strategy, newStrategy);
			 		 break;
		}

		newStrategy = MainThread.adaptSoldiers(newStrategy);
		
		String childStrategy = Arrays.toString(newStrategy).replaceAll("\\[|\\]|,", "");
		Player child = new Player(weakestPlayer.getName(), childStrategy);
		players.remove(weakestPlayer);
		players.add(child);
		selectedPlayers.remove(weakestPlayer);
		selectedPlayers.add(child);
	}
	
	/*
	 * number of soldiers on each tower is average of parents' number on that tower 
	 */
	public static Integer[] crossover1(String[] parent1Strategy, String[] parent2Strategy, Integer[] newStrategy) {
		
		for(int i = 0; i < 10; i++) {
			newStrategy[i] = ((Integer.parseInt(parent1Strategy[i]) + Integer.parseInt(parent2Strategy[i]))/2);
		}
		
		return newStrategy;
	}
	
	/*
	 * single point crossover
	 * point is picked randomly, first part of strategy inherited from first parent, second part from second parent
	 */
	public static Integer[] crossover2(String[] parent1Strategy, String[] parent2Strategy, Integer[] newStrategy) {
		
		int singlePoint = (int)(Math.random()*11);
		
		for(int i = 0; i < singlePoint; i++) {
			newStrategy[i] = Integer.parseInt(parent1Strategy[i]);
		}
		for(int j = singlePoint; j < 10; j++) {
			newStrategy[j] = Integer.parseInt(parent2Strategy[j]);
		}
		
		// adapting number of soldiers
		scale(newStrategy);
		return newStrategy;
	}
	
	/*
	 * number of soldiers on each tower is inherited randomly from one of two parents
	 */
	public static Integer[] crossover3(String[] parent1Strategy, String[] parent2Strategy, Integer[] newStrategy) {
		
		for(int i = 0; i < 10; i++) {
			if(Math.random() < 0.5)
				newStrategy[i] = Integer.parseInt(parent1Strategy[i]);
			else
				newStrategy[i] = Integer.parseInt(parent2Strategy[i]);
		}
		
		scale(newStrategy);
		return newStrategy;
	}
	
	/*
	 * if difference of number of soldiers in new strategy and the specified number is >= 10
	 * that difference is allocated to all towers equally
	 * the remaining < 10 is adapted in Main function adapt soldiers
	 */
	public static Integer[] scale(Integer[] newStrategy) {
		
		int remainder = MainThread.getNumOfSoldiers();
		
		for(int i = 0; i < 10; i++) {
			remainder -= newStrategy[i];
		}
		
		while(remainder >= 10) {
			for(int i = 0; i < 10; i++) {
				newStrategy[i]++;
			}
			remainder -= 10;
		}
		
		while(remainder <= -10) {
			for(int i = 0; i < 10; i++) {
				newStrategy[i]--;
			}
			remainder += 10;
		}
		
		return newStrategy;
	}
	
	/*
	 * two worse players from selected are sent to one of 3 mutation functions
	 */
	public static void mutation(List<Player> players, Set<Player> selectedPlayers, float mutationProbability) {
		
		Player p1 = (Player) selectedPlayers.toArray()[1];
		Player p2 = (Player) selectedPlayers.toArray()[2];
		
		String[] p1Strategy = p1.getStrategy().split(" ");
		String[] p2Strategy = p2.getStrategy().split(" ");
		
		Integer[] mutatedStrategy1 = new Integer[10];
		Integer[] mutatedStrategy2 = new Integer[10];
		
		int mutationType = (int)(Math.random()*3);
		switch(mutationType) {
			case(0): mutatedStrategy1 = mutate1(p1Strategy, mutationProbability);
					 mutatedStrategy2 = mutate1(p2Strategy, mutationProbability);
					 break;
			case(1): mutatedStrategy1 = mutate2(p1Strategy, mutationProbability);
					 mutatedStrategy2 = mutate2(p2Strategy, mutationProbability);
			 		 break;
			case(2): mutatedStrategy1 = mutate3(p1Strategy, mutationProbability);
					 mutatedStrategy2 = mutate3(p2Strategy, mutationProbability);
			 		 break;
		}
		
		String p1mutatedStrategy = Arrays.toString(mutatedStrategy1).replaceAll("\\[|\\]|,", "");
		String p2mutatedStrategy = Arrays.toString(mutatedStrategy2).replaceAll("\\[|\\]|,", "");
		
		Player p1Mutated = new Player(p1.getName(), p1mutatedStrategy);
		Player p2Mutated = new Player(p2.getName(), p2mutatedStrategy);
		
		players.remove(p1);
		players.remove(p2);
		players.add(p1Mutated);
		players.add(p2Mutated);
	}
	
	/*
	 * for each tower, if mutation happens (random number is less than mutation probability)
	 * number of soldiers is changed within -5 to 5 range
	 */
	public static Integer[] mutate1(String[] strategy, float mutationProbability) {
		
		Integer[] mutatedStrategy = new Integer[10];
		
		for(int i = 0; i < 10; i++) {
			double r = Math.random();
			if(r < mutationProbability) {
				int offset = (int)(Math.random()*11) - 5;
				mutatedStrategy[i] = Integer.parseInt(strategy[i]) + offset;
				if(mutatedStrategy[i] > MainThread.getNumOfSoldiers())
					mutatedStrategy[i] = MainThread.getNumOfSoldiers();
				if(mutatedStrategy[i] < 0)
					mutatedStrategy[i] = 0;
			} else {
				mutatedStrategy[i] = Integer.parseInt(strategy[i]);
			}
		}
		
		mutatedStrategy = MainThread.adaptSoldiers(mutatedStrategy);
		
		return mutatedStrategy;
	}
	
	/*
	 * for each pair of towers(2i, 2i+1), if mutation happens (random number is less than mutation probability)
	 * random offset is calculated randomly within -5 to 5 range
	 * first tower is increased by offset, second is decreased
	 */
	public static Integer[] mutate2(String[] strategy, float mutationProbability) {
		
		Integer[] mutatedStrategy = new Integer[10];
		
		for(int i = 0; i < 10; i = i + 2) {
			double r = Math.random();
			if(r < mutationProbability) {
				int offset = (int)(Math.random()*11) - 5;
				mutatedStrategy[i] = Integer.parseInt(strategy[i]) + offset;
				if(mutatedStrategy[i] > MainThread.getNumOfSoldiers())
					mutatedStrategy[i] = MainThread.getNumOfSoldiers();
				if(mutatedStrategy[i] < 0)
					mutatedStrategy[i] = 0;
				mutatedStrategy[i+1] = Integer.parseInt(strategy[i+1]) - offset;
				if(mutatedStrategy[i+1] > MainThread.getNumOfSoldiers())
					mutatedStrategy[i+1] = MainThread.getNumOfSoldiers();
				if(mutatedStrategy[i+1] < 0)
					mutatedStrategy[i+1] = 0;
			} else {
				mutatedStrategy[i] = Integer.parseInt(strategy[i]);
				mutatedStrategy[i+1] = Integer.parseInt(strategy[i+1]);
			}
		}
		
		mutatedStrategy = MainThread.adaptSoldiers(mutatedStrategy);
		
		return mutatedStrategy;
	}
	
	/*
	 * swap mutation
	 * if mutation happens (random number is less than mutation probability)
	 * two towers are randomly picked and number of soldiers on them are swapped
	 */
	public static Integer[] mutate3(String[] strategy, float mutationProbability) {
		
		Integer[] mutatedStrategy = new Integer[10];
		
		for(int i = 0; i < 10; i++)
			mutatedStrategy[i] = Integer.parseInt(strategy[i]);
		
		double r = Math.random();
		if(r < mutationProbability) {
			ArrayList<Integer> list = new ArrayList<Integer>();
	        for(int i = 0; i < 10; i++) {
	            list.add(i);
	        }
	        Collections.shuffle(list);
	        
	        int first = list.get(0);
	        int second = list.get(1);
	        
	        int temp = mutatedStrategy[first];
	        mutatedStrategy[first] = mutatedStrategy[second];
	        mutatedStrategy[second] = temp;
		}
		
		mutatedStrategy = MainThread.adaptSoldiers(mutatedStrategy);
		
		return mutatedStrategy;
	}
}
