package game;

import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


public class GameThread extends Thread{

	private List<Player> players;
	private Map<Player, Integer> results;
	private Map<Player, Integer> resultSorted;
	private Tower tower;
	
	public GameThread(List<Player> players, Tower tower) {
		this.players = players;
		this.tower = tower;
		results = new HashMap<>();
		resultSorted = new HashMap<>();
	}
	
	/*
	 * game
	 * every player battles with each other
	 * for every tower, player with larger number of soldiers on it gets points of its value
	 * if number of soldiers is equal, they split the points
	 */
	@Override
	public void run() {
		players.forEach((p) -> results.put(p, 0));
		
		for(Player p1: players)
			for(Player p2: players) {
				if(p1==p2) continue;

				String[] str1 = p1.getStrategy().split(" ");
				String[] str2 = p2.getStrategy().split(" ");
				
				for(int i = 0; i < 10; i++) {
					try {
						if(Integer.parseInt(str1[i]) > Integer.parseInt(str2[i]))
							results.put(p1, results.get(p1) + tower.getTowerValue(i));
		
						else
						if(Integer.parseInt(str1[i]) < Integer.parseInt(str2[i]))
							results.put(p2, results.get(p2) + tower.getTowerValue(i));
						
						else {
							results.put(p1, results.get(p1) + tower.getTowerValue(i)/2);
							results.put(p2, results.get(p2) + tower.getTowerValue(i)/2);
						}
					} catch(NumberFormatException e) {
						System.out.println(str1[i]);
					}
				}
				
			}
		// divide score by 2 because they play each other 2 times
		results.forEach((name, value) ->  results.put(name, value/2));
		resultSorted = results.entrySet().stream()
                .sorted(Map.Entry.comparingByValue(Comparator.reverseOrder()))
                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue,
                        (oldValue, newValue) -> oldValue, LinkedHashMap::new));
	}
	
	public Map<Player, Integer> getResults(){
		return resultSorted;
	}
}
