package game;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;


public class MainThread extends Thread {
	
	private static String towerValues;
	private static int numOfPlayers;
	private static int numOfSoldiers;
	private static int numOfIterations;
	private static float mutationProbability;
	
	private static Player user;
	
	public void run() {
		
		List<Player> players = new ArrayList<>();
		Scanner sc = new Scanner(System.in);
		
		Tower tower = new Tower(towerValues);
		
		/*
		 * initial generation
		 * each of 3 functions creates strategy to 1/3 of players
		 */
		
		for(int i = 0; i < numOfPlayers/3; i++) {
			Player p = new Player("A"+ (i+1), randomStrategy(numOfSoldiers));
			players.add(p);
		}
		
		for(int i = 0; i < numOfPlayers/3; i++) {
			Player p = new Player("B"+ (i+1), randomStrategy2(numOfSoldiers, tower));
			players.add(p);
		}
		
		for(int i = 0; i < numOfPlayers - 2*(numOfPlayers/3); i++) {
		Player p = new Player("C"+ (i+1), randomStrategy3(numOfSoldiers, tower));
		players.add(p);
		}
	
		
		// starting the game
		GameThread gameThread = new GameThread(players, tower);
		gameThread.run();
				
		int totalNumOfPlayers = numOfPlayers;
		List<Player> newPlayers = new ArrayList<>();

		// doing number of iterations specified
		for(int i = 0; i < numOfIterations; i++) {
			
			Board.setIterationLabel(i + 1);
			
			newPlayers = new ArrayList<>();
			Map<Player, Integer> results = gameThread.getResults();

			int br = 0;
			/*
			 * elimination selection
			 * best 90% of population go through genetic processes
			 * other 10% are replaced by new ones
			 */
			for (Map.Entry<Player, Integer> entry : results.entrySet()) {
				++br;
				newPlayers.add(entry.getKey());
				if(br >= numOfPlayers*0.9)
					break;
			}
			
			for(int j = 0; j < (numOfPlayers-br)/3; j++) {
			Player p = new Player("A" + (i+1) + "_" + (++totalNumOfPlayers), randomStrategy(numOfSoldiers));
			newPlayers.add(p);
			}
			for(int j = 0; j < (numOfPlayers-br)/3; j++) {
				Player p = new Player("B" + (i+1) + "_" + (++totalNumOfPlayers), randomStrategy2(numOfSoldiers, tower));
				newPlayers.add(p);
			}
			for(int j = 0; j < numOfPlayers-br-2*((numOfPlayers-br)/3); j++) {
				Player p = new Player("C" + (i+1) + "_"+ (++totalNumOfPlayers), randomStrategy3(numOfSoldiers, tower));
				newPlayers.add(p);
			}

			/*
			 * genetic processes
			 * number of selections, crossovers and mutations is proportional to number of players
			 * on each iteration we do 10% of that number
			 */
			for(int j = 0; j < numOfPlayers/10; j++) {
				Set<Player> selectedPlayers = GAProcesses.selection(results);			
				GAProcesses.crossover(newPlayers, selectedPlayers);
				GAProcesses.mutation(newPlayers, selectedPlayers, mutationProbability);
			}
			
			// playing the game new iteration of players
			gameThread = new GameThread(newPlayers, tower);
			gameThread.run();
		}
		
		
		if(!(user == null) && Board.isUserPlaying()) {
			newPlayers.add(user);
			gameThread = new GameThread(newPlayers, tower);
			gameThread.run();
		}
		
		Board.finish();
		
		System.out.println("Results");
		int rank = 0;
		for(Map.Entry<Player, Integer> entry: gameThread.getResults().entrySet()) {
			System.out.println("" + ++rank + ".\t" + entry.getKey().getName()+ " : " + entry.getValue() + "\t\t" + entry.getKey().getStrategy());
		}
		new ResultFrame(gameThread.getResults(), players);
		
		sc.close();
	}
	

	
	public static void setUser(Player user) {
		MainThread.user = user;
	}

	public static void setTowerValues(String towerValues) {
		MainThread.towerValues = towerValues;
	}

	public static void setNumOfPlayers(int numOfPlayers) {
		MainThread.numOfPlayers = numOfPlayers;
	}

	public static void setNumOfSoldiers(int numOfSoldiers) {
		MainThread.numOfSoldiers = numOfSoldiers;
	}

	public static void setNumOfIterations(int numOfIterations) {
		MainThread.numOfIterations = numOfIterations;
	}

	public static void setMutationProbability(float mutationProbability) {
		MainThread.mutationProbability = mutationProbability;
	}

	public static int getNumOfSoldiers() {
		return numOfSoldiers;
	}

	/*
	 * completely random strategy
	 * number of soldiers on each tower is random between 0 and 100
	 * then the numbers are scaled so the sum is equal to parameter
	 */
	public static String randomStrategy(int numOfSoldiers) {
		Integer[] randomStrategy = new Integer[10];
		int sum = 0;
		
		for(int i = 0; i < 10; i++) {
			randomStrategy[i] = (int) (Math.random()*100);
			sum += randomStrategy[i];
		}
		
		for(int i = 0; i < 10; i++) {
			randomStrategy[i] = numOfSoldiers*randomStrategy[i]/sum;
		}
		
		// correction due to rounding to integer
		randomStrategy = adaptSoldiers(randomStrategy);

		String strategy = Arrays.toString(randomStrategy).replaceAll("\\[|\\]|,", "");
		
		return strategy;
	}
	
	/*
	 * strategy considering values of towers
	 * number of soldiers on each tower depends on remaining soldiers and tower value compared to other towers
	 * the last tower gets remaining soldiers
	 */
	public static String randomStrategy2(int numOfSoldiers, Tower tower) {
		Integer[] randomStrategy = new Integer[10];
		int remainingSoldiers = numOfSoldiers;
		
		for(int i = 0; i < 10; i++) {
			randomStrategy[i] = (int)(Math.random()*((float)tower.getTowerValue(i)/tower.getTowerSum())*remainingSoldiers*3)%remainingSoldiers;
			remainingSoldiers -= randomStrategy[i];
			if(i == 8) {
				randomStrategy[9] = remainingSoldiers;
				break;
			}
		}
		String strategy = Arrays.toString(randomStrategy).replaceAll("\\[|\\]|,", "");
		
		return strategy;
	}

	/*
	 * strategy assigning soldiers to towers considering remaining soldiers
	 * soldiers are assigned to towers with highest value first, then to others
	 */
	public static String randomStrategy3(int numOfSoldiers, Tower tower) {
		Integer[] randomStrategy = new Integer[10];
		int remainingSoldiers = numOfSoldiers;
		Integer[] sortedTowers = tower.getSortedTowers();
		
		for(int i = 0; i < 10; i++) {
			randomStrategy[sortedTowers[i]] = (int)(Math.random()*remainingSoldiers); 
			remainingSoldiers -= randomStrategy[sortedTowers[i]];
			if(i == 8) {
				randomStrategy[sortedTowers[i+1]] = remainingSoldiers;
				break;
			}
		}
		String strategy = Arrays.toString(randomStrategy).replaceAll("\\[|\\]|,", "");
		
		return strategy;
	}

	/*
	 * function adapts strategy so each would have equal number of soldiers
	 * the remainder is calculated and assigned randomly to towers
	 */
	public static Integer[] adaptSoldiers(Integer[] strategy) {
		
		for(int i = 0; i < 10; i++) {
			if(strategy[i] < 0)
				strategy[i] = 0;
		}
		
		int remainder = numOfSoldiers;
		for(int i = 0; i < 10; i++) {
			remainder -= strategy[i];
		}
		if(remainder > 0) {
			for(int j = 0; j < remainder; j++) {
				int r = (int) (Math.random()*10);
				strategy[r]++;
			}
		} else {
			for(int j = 0; j < -1 * remainder; j++) {
				int r = (int) (Math.random()*10);
				if(strategy[r] > 0)
					strategy[r]--;
				else
					j--;
			}
		}
		
		return strategy;
	}
}
