package game;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Image;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;

/*
 * display of results of the game
 * players' position, name, score and strategy are displayed
 * players are ranked by score
 */
@SuppressWarnings("serial")
public class ResultFrame extends JFrame{

	public ResultFrame(Map<Player, Integer> results, List<Player> players) {
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setVisible(true);
		this.setLocation(950, 50);
		this.setTitle("Results");
		
		JPanel resultPanel = new JPanel();
		add(resultPanel, BorderLayout.CENTER);
		
		//-----------------------------------------------------------
		// player positions on the left side
		JPanel rankPanel = new JPanel();
		resultPanel.add(rankPanel, BorderLayout.WEST);
		rankPanel.setLayout(new GridLayout(0, 1));
		int br = results.size();
		rankPanel.add(new JLabel(""));
		
		for(Integer i = 1; i <= br; i++) {
			JLabel label = new JLabel("   " + i.toString() + "   ", SwingConstants.CENTER);
			label.setBackground(new Color(200,200,250));
			label.setPreferredSize(new Dimension(60,30));
			label.setOpaque(true);
			rankPanel.add(label);
		}
		//----------------------------------------------------------
		
		// name and score in the middle
		JPanel scorePanel = new JPanel();
		resultPanel.add(scorePanel, BorderLayout.CENTER);
		scorePanel.setLayout(new GridLayout(0,2));
		
		JLabel nameLabel = new JLabel("Name", SwingConstants.CENTER);
		JLabel scoreLabel = new JLabel("Score", SwingConstants.CENTER);
		nameLabel.setOpaque(true);
		scoreLabel.setOpaque(true);
		nameLabel.setBackground(new Color(200,200,250));
		scoreLabel.setBackground(new Color(200,200,250));

		scorePanel.add(nameLabel);
		scorePanel.add(scoreLabel);
		
		ImageIcon goldMedal = new ImageIcon(getClass().getResource("goldmedal.jpg"));
		ImageIcon goldMedalScaled = new ImageIcon(goldMedal.getImage().getScaledInstance(30,30, Image.SCALE_DEFAULT));
		
		int counter=0;
		for(Map.Entry<Player, Integer> entry: results.entrySet()) {
			++counter;
			if(counter == 1) {
				JPanel goldenPanel = new JPanel();
				goldenPanel.setLayout(new BorderLayout());
				scorePanel.add(goldenPanel);
				goldenPanel.add(new JLabel(entry.getKey().getName() + " ", SwingConstants.CENTER), BorderLayout.CENTER);
				goldenPanel.add(new JLabel(goldMedalScaled), BorderLayout.EAST);
				scorePanel.add(new JLabel(entry.getValue().toString(), SwingConstants.CENTER));
			}
			else {
				scorePanel.add(new JLabel(entry.getKey().getName(), SwingConstants.CENTER));
				scorePanel.add(new JLabel(entry.getValue().toString(), SwingConstants.CENTER));
			}
		}

		// adding scroll
		JScrollPane scrollFrame = new JScrollPane(resultPanel);
		this.add(scrollFrame);
		
		//---------------------------------------------------------
		// player strategies on the right side
		JPanel strategiesPanel = new JPanel();
		resultPanel.add(strategiesPanel, BorderLayout.EAST);
		strategiesPanel.setLayout(new GridLayout(0,1));
		
		JLabel strategyLabel = new JLabel("Strategy", SwingConstants.CENTER);
		strategyLabel.setOpaque(true);
		strategyLabel.setBackground(new Color(200,200,250));
		strategyLabel.setPreferredSize(new Dimension(180,30));
		strategiesPanel.add(strategyLabel, SwingConstants.CENTER);

		for(Player player: results.keySet()) {
			strategiesPanel.add(new JLabel(player.getStrategy(), SwingConstants.CENTER));
		}
		
		if(players.size() > 22)
			this.setSize(450, 750);
		else
			this.setSize(450, 30 * (players.size()+3));
	}
}